/*
 * main.c
 *
 *  Created on: 07/mag/2012
 *      Author: fb
 */

#include <stdio.h>
#include "system.h"
#include "alt_types.h"
#include "sys/alt_timestamp.h"
#include <sys/alt_irq.h>
#include "data.h"


/* define macros to access parallel DE2 Basic Computer port registers */
#define EDGE_CAPTURE_REG_FFSET		3

#define RED_LEDS_DATA_REG 			(*(volatile unsigned int*) RED_LEDS_BASE)
#define GREEN_LEDS_DATA_REG 		(*(volatile unsigned int*) GREEN_LEDS_BASE)
#define HEX3_HEX0_DATA_REG 			(*(volatile unsigned int*) HEX3_HEX0_BASE)
#define HEX7_HEX4_DATA_REG 			(*(volatile unsigned int*) HEX7_HEX4_BASE)
#define PUHBUTTONS_DATA_REG 		(*(volatile unsigned int*) PUSHBUTTONS_BASE)
#define PUHBUTTONS_EDGE_CAPTURE_REG	(*(volatile unsigned int*) (PUSHBUTTONS_BASE + 4*EDGE_CAPTURE_REG_FFSET))
#define SLIDER_SWITCHES_DATA_REG	(*(volatile unsigned int*) SLIDER_SWITCHES_BASE)

/* ------------------------------------------------------------------------- */

/*
 *   Seven segment
 *         0
 *       -----
 *     5 |___| 1
 *     4 | 6 | 2
 *       -----
 *         3
*/

const unsigned char segments[16] = {
    0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7C, 0x07, 0x7F, 0x67, /* 0-9 */
    0x77, 0x7C, 0x39, 0x5E, 0x79, 0x71 };                       /* A-F */

unsigned int hex_to_sevenseg(unsigned int hex) {
	unsigned int data = segments[hex & 15] | (segments[(hex >> 4) & 15] << 8) |
    		           (segments[(hex >> 8) & 15] << 16) | (segments[(hex >> 12) & 15] << 24);
    return data;
}
/* ------------------------------------------------------------------------- */

/* delay generation */
#define CYCLES_PER_MS 254 /* hand tuned to achieve around 1 ms resolution (code optimization OFF)*/
void wait_ms(unsigned int time_ms) {
	int i,j;
	for(j=0; j<time_ms; j++) {
		for(i=0; i<CYCLES_PER_MS; i++) {;}
	}
}
/* ------------------------------------------------------------------------- */


#define COUNT	256

/*
int a[COUNT] = {A_ARRAY};
int x[COUNT] = {X_ARRAY};
volatile int y;
*/

int a[COUNT] __attribute__ ((section (".tightly.rwdata"))) = {A_ARRAY};
int x[COUNT] __attribute__ ((section (".tightly.rwdata"))) = {X_ARRAY};
volatile int y __attribute__ ((section (".tightly.rwdata"))) = 0;


int dotp(int *m, int *n, int count) __attribute__ ((section (".tightly.text")));

int dotp(int *m, int *n, int count)
{ int i;
  int sum = 0;

  for (i=0; i < count; i++)
  {
    sum = sum + m[i] * n[i];
  }
  return(sum);
}

#define NUM_ITER	10

int main() {

	alt_u32 time1;
	alt_u32 time2;
	alt_u32 time[NUM_ITER];

	if (alt_timestamp_start() < 0) {
		printf ("No timestamp device available\n");
		return -1;
	}
	//printf("Interval Timer clock frequency = %uHz\n", (unsigned int) alt_timestamp_freq());

	//alt_irq_context irq_context;
	//irq_context = alt_irq_disable_all();

	int i;
	for(i=0; i<NUM_ITER; i++) {
		alt_timestamp_start();
		time1 = alt_timestamp();
		y = dotp(a, x, COUNT);
		time2 = alt_timestamp();
		time[i] = time2 - time1;
	}

	//alt_irq_enable_all(irq_context);

	printf("y = %x hex (%d decimal)\n", y, y);

	for(i=0; i<NUM_ITER; i++) {
		printf("ITER %d : Execution Time = %d cycles\n", i, (int)time[i]);
	}

	return 0;
}// main
